@@ -44,24 +44,37 @@
 #include "PeerStorage.h"
 #include "Peer.h"
 #include "Logger.h"
+#include "LogFactory.h"
 #include "bittorrent_helper.h"
 #include "DownloadContext.h"
 #include "wallclock.h"
-#include "RequestGroupMan.h"
-#include "FileAllocationEntry.h"
-#include "CheckIntegrityEntry.h"
-#include "ServerStatMan.h"
+#include "fmt.h"
 
 namespace aria2 {
 
-DHTGetPeersCommand::DHTGetPeersCommand(cuid_t cuid,
-                                       RequestGroup* requestGroup,
-                                       DownloadEngine* e):
-  Command(cuid),
-  requestGroup_(requestGroup),
-  e_(e),
-  numRetry_(0),
-  lastGetPeerTime_(0)
+namespace {
+
+const time_t GET_PEER_INTERVAL = (15*60);
+// Interval when the size of peer list is low.
+const time_t GET_PEER_INTERVAL_LOW = (5*60);
+// Interval when the peer list is empty.
+const time_t GET_PEER_INTERVAL_ZERO = 60;
+// Interval for retry.
+const time_t GET_PEER_INTERVAL_RETRY = 5;
+// Maximum retries. Try more than 5 to drop bad node.
+const size_t MAX_RETRIES = 10;
+
+} // namespace
+
+DHTGetPeersCommand::DHTGetPeersCommand
+(cuid_t cuid,
+ RequestGroup* requestGroup,
+ DownloadEngine* e)
+  : Command(cuid),
+    requestGroup_(requestGroup),
+    e_(e),
+    numRetry_(0),
+    lastGetPeerTime_(0)
 {
   requestGroup_->increaseNumCommand();
 }
@@ -76,25 +89,28 @@ bool DHTGetPeersCommand::execute()
   if(btRuntime_->isHalt()) {
     return true;
   }
-  if(task_.isNull() &&
-     ((numRetry_ > 0 &&
-       lastGetPeerTime_.difference(global::wallclock) >= (time_t)numRetry_*5) ||
-      lastGetPeerTime_.difference(global::wallclock) >= GET_PEER_INTERVAL ||
-      (btRuntime_->lessThanMinPeers() &&
-       lastGetPeerTime_.difference(global::wallclock) >= GET_PEER_MIN_INTERVAL
+  time_t elapsed = lastGetPeerTime_.difference(global::wallclock);
+  if(!task_ &&
+     (elapsed >= GET_PEER_INTERVAL ||
+      (((btRuntime_->lessThanMinPeers() &&
+         ((numRetry_ && elapsed >= GET_PEER_INTERVAL_RETRY) ||
+          elapsed >= GET_PEER_INTERVAL_LOW)) ||
+        (btRuntime_->getConnections() == 0 &&
+         elapsed >= GET_PEER_INTERVAL_ZERO))
        && !requestGroup_->downloadFinished()))) {
-    if(getLogger()->debug()) {
-      getLogger()->debug("Issuing PeerLookup for infoHash=%s",
-                         bittorrent::getInfoHashString
-                         (requestGroup_->getDownloadContext()).c_str());
-    }
+    A2_LOG_DEBUG(fmt("Issuing PeerLookup for infoHash=%s",
+                     bittorrent::getInfoHashString
+                     (requestGroup_->getDownloadContext()).c_str()));
     task_ = taskFactory_->createPeerLookupTask
       (requestGroup_->getDownloadContext(), btRuntime_, peerStorage_);
     taskQueue_->addPeriodicTask2(task_);
-  } else if(!task_.isNull() && task_->finished()) {
+  } else if(task_ && task_->finished()) {
+    A2_LOG_DEBUG("task finished detected");
     lastGetPeerTime_ = global::wallclock;
     if(numRetry_ < MAX_RETRIES && btRuntime_->lessThanMinPeers()) {
       ++numRetry_;
+      A2_LOG_DEBUG(fmt("Too few peers. Try again(%lu)",
+                       static_cast<unsigned long>(numRetry_)));
     } else {
       numRetry_ = 0;
     }
